package state51::Logger::Event;
use Moose;
use MooseX::Storage;
use MooseX::Types::Moose qw/Int Str Undef/;
use Data::UUID;
use MooseX::Getopt;
use namespace::clean -except => 'meta';

with Storage('format' => 'JSON', 'io' => 'File');

# should be overridden, but we never want his method to not exist
sub stringify { ($_[0]->can('message') && $_[0]->message) || 'Empty message from abstract base event' }

with 'state51::Log::Message::Structured';

# for t/instantiation.t
sub as_string { &stringify }

sub routing_key_filter {
    my $class = blessed( $_[0] ) ? blessed( $_[0] ) : $_[0];
    $class =~ s/::/_/g;
    $class = lc($class);
    "state51.logger.*.*.*.*.*.$class.#";
}

sub append_routing_key {
    my $self = shift;
    my $class = lc ref($self);
    $class =~ s/::/_/g;
    sprintf("%s.%s.%s.%s",
        $self->pid,
        $self->hitid,
        $class,
        $self->UUID
    );
}

my $ug = Data::UUID->new;
has UUID => (
    is => 'ro',
    default => sub { my $uuid = $ug->create; $ug->to_string($uuid) },
    traits => [qw/ NoGetopt /],
    documentation => 'A unique id for the event',
);

has EventName => (
    isa => Str,
    is => 'ro',
    default => sub { my $class = blessed( $_[0] ) ? blessed( $_[0] ) : $_[0]; $class =~ s/^state51::Logger::Event:://; $class =~ s/::/\_/; return $class; },
    documentation => 'This is the event name with `Event::` removed which makes it easier to search for in kibana',
);

has OriginatingOrganisationID => (
    isa => Int,
    is => 'ro',
    default => 8632477670016,
);

has ApplicationID => (
    isa => Int,
    is => 'ro',
    default => 54838309120320,
    documentation => 'The instance id of the `state51::Application` raising the event',
);

has Environment => (
    isa => Str,
    is => 'ro',
    default => 'production',
    documentation => 'The environment the event is coming from, defaults to `production`.',
);

has pid => (
    isa => Int, is => 'ro', default => sub { $$;  }, required => 1,
    traits => [qw/ NoGetopt /],
    init_arg => undef,
);

has raised_by_org => (
    isa => 'Int|Undef',
    is  => 'ro',
    default => undef,
    required => 0,
);

# Note, we DO NOT use state51::Handler::Base here, otherwise
#       we create recursive dependencies and thing go weird.
has hitid => (
    isa => Str,
    is => 'ro',
    default => sub {
        no strict 'vars';
        'nonweb';
        use strict;
    },

    traits => [qw/ NoGetopt /],
);

has logger_version => (
    isa => Str,
    is => 'ro',
    default => sub { return $state51::Logger::VERSION || 'Logger version unknown' },
);

has platform_version => (
    isa => Str,
    is => 'ro',
    #init_arg => ' ',
    default => sub {
        eval  { require state51::Base };
        if($@) {
            return ' ';
        }
        else {

            return $state51::Base::VERSION || 'platform version unknown';
        }
    },
);

# Why does MX::Storage decode JSON, it's outputting, that's insane!!!
around freeze => sub {
    my ($orig, $self, @args) = @_;
    my $json = $self->$orig(@args);
    utf8::encode($json);
    return $json;
};

__PACKAGE__->meta->make_immutable;
1;
