package state51::Logger::Event::AttributeChanged;
use Moose;
use MooseX::Types::Moose qw/Int Str/;
use state51::Types qw/InstanceID/;
use MooseX::ClassAttribute;
use namespace::autoclean;

extends 'state51::Logger::Event';

class_has documentation => (
    is  => 'ro',
    isa => Str,
    default =>
      'Called when an attribute (e.g. status) is changed on an object. '
      . 'This is called as the attribute is changed, not when its persisted.'
      . " This may mean that events here don't match the persistent object state, "
      . 'but will mean that we capture every change event during the lifespan of the instance.',
);

has attribute_name => (
    isa      => 'Str',
    is       => 'rw',
    required => 1,
);

has previous_value => (
    isa      => 'Str',
    is       => 'rw',
    required => 1,
);

has current_value => (
    isa      => 'Str',
    is       => 'rw',
    required => 1,
);

has class => (
    isa      => 'Str',
    is       => 'rw',
    required => 1,
);

has instance_id => (
    isa      => InstanceID,
    is       => 'rw',
    required => 1,
);

with 'state51::Log::Message::Structured::Stringify::Sprintf' => {
    format_string => q{%s with id %d changed attribute %s from %s to %s},
    attributes =>
      [qw/ class instance_id attribute_name previous_value current_value/],
};

__PACKAGE__->meta->make_immutable;

