package state51::Logger::Event::Environment;
use Moose;
use MooseX::Types::Moose qw/ Int Str Num /;
use MooseX::ClassAttribute;
use namespace::clean -except => 'meta';

extends 'state51::Logger::Event';

class_has documentation => (
    is      => 'ro',
    isa     => Str,
    default => 'This event contains environmental sensor data from room monitoring devices including temperature, humidity and air quality metrics.',
);

has [qw/ hostname building_name room_name /] => (
    isa => Str,
    is => 'ro',
    required => 1,
);

# Temperature/humidity sensor attributes
has [qw/ temperature relative_humidity /] => (
    isa => Num,
    is => 'ro',
    required => 0,
);

has [qw/ temp_sensor_serial_number temp_sensor_model /] => (
    isa => Str,
    is => 'ro',
    required => 0,
);

# Air quality sensor attributes
has [qw/ pm10_standard pm25_standard pm100_standard
        pm10_env pm25_env pm100_env /] => (
    isa => Num,
    is => 'ro',
    required => 0,
);

has [qw/ particles_03um particles_05um particles_10um
        particles_25um particles_50um particles_100um /] => (
    isa => Int,
    is => 'ro',
    required => 0,
);

# Add stringify format for logging
with 'state51::Log::Message::Structured::Stringify::Sprintf' => {
    format_string => q{Environment data for %s in %s %s: %.1f°C, %.1f%% RH, PM2.5: %.1f µg/m³},
    attributes => [qw/ hostname building_name room_name temperature relative_humidity pm25_env /],
};

__PACKAGE__->meta->make_immutable;
1;
