package state51::Logger::Event::Greedbag::StockMovement;
use Moose;
use MooseX::Storage ();
use MooseX::Types::Moose qw/ Num Str Int Maybe /;
use state51::Types qw/ InstanceID PackageName /;
use namespace::autoclean;

extends 'state51::Logger::Event::Greedbag';

sub routing_key { 'state51.greedbag.stockmovement' }

=head1 Attributes

=head2

C<state51::Logger::Event::Greedbag> provide the following:

=over 2

=item C<retailer_name>

=item C<retailer_id>

=back

=cut

has new_quantity => (
    is       => 'ro',
    isa      => Num,
    required => 1,
);

has existing_quantity => (
    is       => 'ro',
    isa      => Num,
    required => 1,
);

=head2 caller

Theorically, it should only be Platform's C<state51::StockManagement::Types>
classes. Not really the role of a logger to validate that, though.

=cut

has caller => (
    isa      => PackageName,
    required => 1,
    is       => 'ro',
);

has instance => (
    is       => 'ro',
    isa      => Maybe [InstanceID],
    required => 0,
);

# context? carp? caller?

with 'state51::Log::Message::Structured::Stringify::Sprintf' => {
    format_string => q{Stock movement %s %s %s},
    attributes => [qw/ existing_quantity new_quantity caller /],
};

__PACKAGE__->meta->make_immutable;
1;

