package state51::Logger::Event::ScriptRun;
use Moose;
use namespace::clean -except => 'meta';

extends 'state51::Logger::Event';

has 'name' => (
    is => 'ro',
    default => sub {
        my $name = $0;
        $name = "oneliner" if $name eq '-e';
        $name =~ s!^.*/!!;
        $name;
    },
);

has user => (
    is => 'ro',
    default => sub {
        my ($uid) = getpwuid($<);
        return $uid;
    },
);

has 'time' => (
    is => 'ro',
    default => sub {
        time() - $state51::Base::START_TIME;
    },
);

has queries => (
    is => 'ro',
    default => sub {
        eval { DBI::db::_get_query_count() };
    },
);

has classes_cached => (
    is      => 'ro',
    default => sub {
        join(',', @{state51::Base->return_classes_in_cache()});
    },
);

with 'state51::Log::Message::Structured::Stringify::Sprintf' => {
    format_string => q{Script %s took %ss, did %s queries, classes cached: %s},
    attributes => [qw/ name time queries classes_cached/],
};

__PACKAGE__->meta->make_immutable;

