package state51::Logger::Role;
use Moose::Role;
use Moose::Autobox;
use Encode;
use aliased 'state51::Logger::Event::Generic' => 'Event';
use namespace::autoclean;

sub message {
    my ($self, $logger, $args) = @_;

    my @receipts;
    foreach my $message ($args->{message}->flatten) {
        my $receipt = $self->message_line($logger, $args, $message);
        push @receipts,$receipt if $receipt;
    }

    return \@receipts; # log-message event ids
}

sub default_exchange { 'firehose' }

sub message_line {
    my ($self, $logger, $args, $line) = @_;
    my $hostname = $logger->hostname;
    my $cmd = $logger->cmd;
    $hostname =~ s/\./_/g;
    $cmd =~ s/\./_/g;
    $cmd =~ s/_pl//;
    my $event; # don't be misled, this could also be a state51::Job
    if (blessed $line && $line->can('freeze')) {
        $event = $line;
    }
    else {
        if (my $class = blessed $line) {
            warn "$class object doesn't implement 'freeze()' - it should probably 'use MooseX::Storage; with Storage ('format' => 'JSON');'";
            # hope object implements stringification via overload.pm
            $line = "$line";
        }
        # can't even test '$line' in case of
        # "Modification of a read-only value attempted at (eval 1727) line 14."
        $event = Event->new(message => Encode::encode('utf8', $line));
    }
    my $routing_key =
        $event->can('routing_key')
            ? $event->routing_key
            : sprintf('state51.logger.%s.%s.%s',
                $args->{level},
                $hostname,
                $cmd
              )
              . (
                    $event->can('append_routing_key')
                        ? ('.' . $event->append_routing_key)
                        : ''
              );
    my $exchange_name = $event->can('exchange_name')
        ? $event->exchange_name
        : $self->default_exchange;
    $self->publish_message(
            body => $event,
            exchange => $exchange_name,
            routing_key => $routing_key,
    );

    if ($event->can('UUID')) {
        return $event->UUID;
    }
    else {
        return undef;
    }
}

1;
